<?php

namespace Drupal\shopify_app;

use Shopify\Clients\Graphql;

/**
 * Shopify function helpers.
 */
class ShopifyFunctionsHelper {

  /**
   * Get all defined shopify functions.
   *
   * @param \Drupal\shopify_app\ShopInterface $shop
   *   The shop
   *
   * @return array
   *   The defined functions.
   *
   * @throws \JsonException
   * @throws \Shopify\Exception\HttpRequestException
   * @throws \Shopify\Exception\MissingArgumentException
   */
  public function getAllDefined(ShopInterface $shop): array {
    shopify_app_initialize_context();

    $session = SessionStorage::getInstance()
      ->getOfflineSession($shop->id());

    $shopify_graphql_client = new Graphql($session->getShop(), $session->getAccessToken());

    $query = '
      query {
        shopifyFunctions(first: 250) {
          nodes {
            app {
              title
            }
            apiType
            title
            id
          }
        }
      }
    ';

    $response = $shopify_graphql_client->query($query);
    $body = $response->getDecodedBody();

    $shopify_functions = [];

    foreach ($body['data']['shopifyFunctions']['nodes'] as $shopify_function) {
      $shopify_functions[$shopify_function['id']] = [
        'app_title' => $shopify_function['app']['title'],
        'title' => $shopify_function['title'],
        'apiType' => $shopify_function['apiType'],
      ];
    }

    return $shopify_functions;
  }

  /**
   * Create a cart transformation.
   *
   * @param \Drupal\shopify_app\ShopInterface $shop
   *   The shop
   * @param string $shopify_function_id
   *   The shopify function helper id
   *
   * @return string
   *   The gid under which it has been created.
   *
   * @throws \JsonException
   * @throws \Shopify\Exception\HttpRequestException
   * @throws \Shopify\Exception\MissingArgumentException
   */
  public function createCartTransformation(ShopInterface $shop, string $shopify_function_id): string {
    shopify_app_initialize_context();

    $session = SessionStorage::getInstance()
      ->getOfflineSession($shop->id());

    $shopify_graphql_client = new Graphql($session->getShop(), $session->getAccessToken());

    $query = '
      mutation {
        cartTransformCreate(functionId: "' . $shopify_function_id . '") {
          cartTransform {
            id
            functionId
          }
          userErrors {
            field
            message
           }
        }
      }
    ';

    $response = $shopify_graphql_client->query($query);

    $decoded_body = $response->getDecodedBody();
    if (!empty($decoded_body['data']['cartTransformCreate']['userErrors'][0])) {
      throw new \RuntimeException($decoded_body['data']['cartTransformCreate']['userErrors'][0]['message']);
    }

    return $decoded_body['data']['cartTransformCreate']['cartTransform']['id'];
  }

}
